import * as addPlugin from '@graphql-codegen/add';
import * as typedDocumentNodePlugin from '@graphql-codegen/typed-document-node';
import * as typescriptOperationPlugin from '@graphql-codegen/typescript-operations';
import * as typescriptPlugin from '@graphql-codegen/typescript';
import crypto from 'crypto';

const plugin = (schema, sources, { sourcesWithOperations }) => [
    `import * as graphql from './graphql';\n`,
    `\n`,
    ...getDocumentRegistryChunk(sourcesWithOperations),
    `\n`,
    ...getGqlOverloadChunk(sourcesWithOperations),
    `\n`,
    `export function gql(source: string): unknown;\n`,
    `export function gql(source: string) {\n`,
    `  return (documents as any)[source] ?? {};\n`,
    `}\n`,
].join(``);
function getDocumentRegistryChunk(sourcesWithOperations) {
    const lines = [];
    lines.push(`const documents = {\n`);
    for (const { source: { rawSDL }, operations } of sourcesWithOperations) {
        lines.push(`  ${JSON.stringify(rawSDL)}: {\n`);
        for (const { initialName, uniqueName } of operations)
            lines.push(`    ${initialName}: graphql.${uniqueName}Document,\n`);
        lines.push(`  },\n`);
    }
    lines.push(`};\n`);
    return lines;
}
function getGqlOverloadChunk(sourcesWithOperations) {
    const lines = [];
    for (const { source: { rawSDL } } of sourcesWithOperations)
        lines.push(`export function gql(source: ${JSON.stringify(rawSDL)}): (typeof documents)[${JSON.stringify(rawSDL)}];\n`);
    return lines;
}

var dtsGenPlugin = /*#__PURE__*/Object.freeze({
    __proto__: null,
    plugin: plugin
});

function processSources(sources) {
    var _a, _b;
    const fullHashes = new Map();
    for (const source of sources) {
        if (!source)
            continue;
        const hash = crypto.createHash(`sha256`).update(source.rawSDL).digest(`hex`);
        if (fullHashes.has(hash))
            continue;
        fullHashes.set(hash, source);
    }
    let hashLength = 0;
    findHashSize: for (let t = 1; t <= 32; ++t) {
        const seen = new Set();
        for (const hash of fullHashes.keys()) {
            const sub = hash.substr(0, t);
            if (seen.has(sub)) {
                continue findHashSize;
            }
            else {
                seen.add(sub);
            }
        }
        hashLength = t;
        break;
    }
    const subHashes = new Map();
    for (const [hash, record] of fullHashes)
        subHashes.set(hash.substr(0, hashLength), record);
    const sourcesWithOperations = [];
    for (const [hash, source] of subHashes) {
        const { document } = source;
        const operations = [];
        for (const definition of (_a = document === null || document === void 0 ? void 0 : document.definitions) !== null && _a !== void 0 ? _a : []) {
            if ((definition === null || definition === void 0 ? void 0 : definition.kind) !== `OperationDefinition`)
                continue;
            if (((_b = definition.name) === null || _b === void 0 ? void 0 : _b.kind) !== `Name`)
                continue;
            const initialName = definition.name.value;
            const uniqueName = `${initialName}_${hash}_`;
            Object.assign(definition.name, { value: uniqueName });
            operations.push({
                initialName,
                uniqueName,
                definition,
            });
        }
        if (operations.length === 0)
            continue;
        sourcesWithOperations.push({
            source,
            operations,
        });
    }
    return sourcesWithOperations;
}

const preset = {
    buildGeneratesSection: options => {
        var _a;
        const packageName = (_a = options.presetConfig.packageName) !== null && _a !== void 0 ? _a : `@app/gql`;
        const sourcesWithOperations = processSources(options.documents);
        const sources = sourcesWithOperations.map(({ source }) => source);
        const pluginMap = Object.assign(Object.assign({}, options.pluginMap), { [`add`]: addPlugin, [`typescript`]: typescriptPlugin, [`typescript-operations`]: typescriptOperationPlugin, [`typed-document-node`]: typedDocumentNodePlugin, [`gen-dts`]: dtsGenPlugin });
        const plugins = [
            { [`add`]: { content: `/* eslint-disable */` } },
            { [`typescript`]: {} },
            { [`typescript-operations`]: {} },
            { [`typed-document-node`]: {} },
            ...options.plugins,
        ];
        const genDtsPlugins = [
            { [`add`]: { content: `/* eslint-disable */` } },
            { [`gen-dts`]: { sourcesWithOperations, packageName } },
        ];
        return [{
                filename: `${options.baseOutputDir}/graphql.ts`,
                plugins,
                pluginMap,
                schema: options.schema,
                config: options.config,
                documents: sources,
            }, {
                filename: `${options.baseOutputDir}/index.ts`,
                plugins: genDtsPlugins,
                pluginMap,
                schema: options.schema,
                config: options.config,
                documents: sources,
            }];
    },
};

export { preset };
